@@ -24,7 +24,7 @@ LOG.debug(f'Arch: {name} / {platform} / {" / ".join(architecture())}')
 LOG.debug(f'Python: {python_implementation()}-{python_version()}')
 LOG.debug(f'{uname()}')
 
-font_name = 'DejaVuSansMono.ttf'
+FONT_NAME = 'DejaVuSansMono.ttf'
 if platform == 'win32':
-    font_name = 'consola.ttf'
-FONT = {size: ImageFont.truetype(font_name, size) for size in (11, 16, 22, 32)}
+    FONT_NAME = 'consola.ttf'
+FONT = {size: ImageFont.truetype(FONT_NAME, size) for size in (11, 16, 22, 32)}
