@@ -1,7 +1,8 @@
 from functools import partial
+from itertools import chain, cycle
 from logging import getLogger
 from string import whitespace
-from typing import Dict, Union, Tuple
+from typing import Dict, Union, Tuple, Optional, Iterator
 
 from PIL import Image, ImageDraw
 
@@ -12,7 +13,7 @@ try:
 except ImportError:
     from typing import TypedDict
 
-BIOS_VALUE = TypedDict('BIOS_VALUE', {'class': str, 'args': Dict[str, int], 'value': Union[int, str]})
+BIOS_VALUE = TypedDict('BIOS_VALUE', {'class': str, 'args': Dict[str, int], 'value': Union[int, str], 'max_value': int}, total=False)
 LOG = getLogger(__name__)
 
 
@@ -25,6 +26,7 @@ class Aircraft:
         """
         self.lcd = lcd_type
         self.bios_data: Dict[str, BIOS_VALUE] = {}
+        self.cycle_buttons: Dict[str, Iterator[int]] = {}
 
     def button_request(self, button: int, request: str = '\n') -> str:
         """
@@ -51,7 +53,7 @@ class Aircraft:
         """Update display."""
         lcd_sdk.update_display(image)
 
-    def prepare_image(self) -> Image.Image:
+    def prepare_image(self) -> Optional[Image.Image]:
         """
         Prepare image to be send to correct type of LCD.
 
@@ -67,6 +69,7 @@ class Aircraft:
             return img
         except KeyError as err:
             LOG.debug(f'Wrong LCD type: {self.lcd} or key: {err}')
+            return None
 
     def set_bios(self, selector: str, value: str) -> None:
         """
@@ -99,6 +102,22 @@ class Aircraft:
         """Prepare image for Aircraft for Color LCD."""
         raise NotImplementedError
 
+    def get_next_value_for_button(self, btn_name: str) -> int:
+        """
+        Get next int value (cycle fore and back) for button name.
+
+        :param btn_name: BIOS button name
+        :return: next int value
+        """
+        curr_val = int(self.get_bios(btn_name))
+        max_val = self.bios_data[btn_name]['max_value']
+        if not self.cycle_buttons[btn_name]:
+            full_seed = list(range(max_val + 1)) + list(range(max_val - 1, 0, -1)) + list(range(max_val + 1))
+            seed = full_seed[curr_val + 1:2 * max_val + curr_val + 1]
+            LOG.debug(f'{self.__class__.__name__} {btn_name} full_seed: {full_seed} seed: {seed} curr_val: {curr_val}')
+            self.cycle_buttons[btn_name] = cycle(chain(seed))
+        return next(self.cycle_buttons[btn_name])
+
 
 class FA18Chornet(Aircraft):
     def __init__(self, lcd_type: LcdSize) -> None:
@@ -109,56 +128,58 @@ class FA18Chornet(Aircraft):
         """
         super().__init__(lcd_type)
         self.bios_data: Dict[str, BIOS_VALUE] = {
-            'ScratchpadStr1': {'class': 'StringBuffer', 'args': {'address': 0x744e, 'length': 2}, 'value': str()},
-            'ScratchpadStr2': {'class': 'StringBuffer', 'args': {'address': 0x7450, 'length': 2}, 'value': str()},
-            'ScratchpadNum': {'class': 'StringBuffer', 'args': {'address': 0x7446, 'length': 8}, 'value': str()},
-            'OptionDisplay1': {'class': 'StringBuffer', 'args': {'address': 0x7432, 'length': 4}, 'value': str()},
-            'OptionDisplay2': {'class': 'StringBuffer', 'args': {'address': 0x7436, 'length': 4}, 'value': str()},
-            'OptionDisplay3': {'class': 'StringBuffer', 'args': {'address': 0x743a, 'length': 4}, 'value': str()},
-            'OptionDisplay4': {'class': 'StringBuffer', 'args': {'address': 0x743e, 'length': 4}, 'value': str()},
-            'OptionDisplay5': {'class': 'StringBuffer', 'args': {'address': 0x7442, 'length': 4}, 'value': str()},
-            'COMM1': {'class': 'StringBuffer', 'args': {'address': 0x7424, 'length': 2}, 'value': str()},
-            'COMM2': {'class': 'StringBuffer', 'args': {'address': 0x7426, 'length': 2}, 'value': str()},
-            'OptionCueing1': {'class': 'StringBuffer', 'args': {'address': 0x7428, 'length': 1}, 'value': str()},
-            'OptionCueing2': {'class': 'StringBuffer', 'args': {'address': 0x742a, 'length': 1}, 'value': str()},
-            'OptionCueing3': {'class': 'StringBuffer', 'args': {'address': 0x742c, 'length': 1}, 'value': str()},
-            'OptionCueing4': {'class': 'StringBuffer', 'args': {'address': 0x742e, 'length': 1}, 'value': str()},
-            'OptionCueing5': {'class': 'StringBuffer', 'args': {'address': 0x7430, 'length': 1}, 'value': str()},
-            'IFEI_FUEL_DOWN': {'class': 'StringBuffer', 'args': {'address': 0x748a, 'length': 6}, 'value': str()},
-            'IFEI_FUEL_UP': {'class': 'StringBuffer', 'args': {'address': 0x7490, 'length': 6}, 'value': str()}}
+            'UFC_SCRATCHPAD_STRING_1_DISPLAY': {'class': 'StringBuffer', 'args': {'address': 0x744e, 'max_length': 2}, 'value': str()},
+            'UFC_SCRATCHPAD_STRING_2_DISPLAY': {'class': 'StringBuffer', 'args': {'address': 0x7450, 'max_length': 2}, 'value': str()},
+            'UFC_SCRATCHPAD_NUMBER_DISPLAY': {'class': 'StringBuffer', 'args': {'address': 0x7446, 'max_length': 8}, 'value': str()},
+            'UFC_OPTION_DISPLAY_1': {'class': 'StringBuffer', 'args': {'address': 0x7432, 'max_length': 4}, 'value': str()},
+            'UFC_OPTION_DISPLAY_2': {'class': 'StringBuffer', 'args': {'address': 0x7436, 'max_length': 4}, 'value': str()},
+            'UFC_OPTION_DISPLAY_3': {'class': 'StringBuffer', 'args': {'address': 0x743a, 'max_length': 4}, 'value': str()},
+            'UFC_OPTION_DISPLAY_4': {'class': 'StringBuffer', 'args': {'address': 0x743e, 'max_length': 4}, 'value': str()},
+            'UFC_OPTION_DISPLAY_5': {'class': 'StringBuffer', 'args': {'address': 0x7442, 'max_length': 4}, 'value': str()},
+            'UFC_COMM1_DISPLAY': {'class': 'StringBuffer', 'args': {'address': 0x7424, 'max_length': 2}, 'value': str()},
+            'UFC_COMM2_DISPLAY': {'class': 'StringBuffer', 'args': {'address': 0x7426, 'max_length': 2}, 'value': str()},
+            'UFC_OPTION_CUEING_1': {'class': 'StringBuffer', 'args': {'address': 0x7428, 'max_length': 1}, 'value': str()},
+            'UFC_OPTION_CUEING_2': {'class': 'StringBuffer', 'args': {'address': 0x742a, 'max_length': 1}, 'value': str()},
+            'UFC_OPTION_CUEING_3': {'class': 'StringBuffer', 'args': {'address': 0x742c, 'max_length': 1}, 'value': str()},
+            'UFC_OPTION_CUEING_4': {'class': 'StringBuffer', 'args': {'address': 0x742e, 'max_length': 1}, 'value': str()},
+            'UFC_OPTION_CUEING_5': {'class': 'StringBuffer', 'args': {'address': 0x7430, 'max_length': 1}, 'value': str()},
+            'IFEI_FUEL_DOWN': {'class': 'StringBuffer', 'args': {'address': 0x748a, 'max_length': 6}, 'value': str()},
+            'IFEI_FUEL_UP': {'class': 'StringBuffer', 'args': {'address': 0x7490, 'max_length': 6}, 'value': str()}}
+
+    def _draw_common_data(self, draw: ImageDraw, foreground: Union[int, Tuple[int, int, int, int]],
+                          background: Union[int, Tuple[int, int, int, int]], scale: int) -> ImageDraw:
+        scratch_1 = self.get_bios("UFC_SCRATCHPAD_STRING_1_DISPLAY")
+        scratch_2 = self.get_bios("UFC_SCRATCHPAD_STRING_2_DISPLAY")
+        scratch_num = self.get_bios("UFC_SCRATCHPAD_NUMBER_DISPLAY")
+        draw.text(xy=(0, 0), fill=foreground, font=FONT[16 * scale],
+                  text=f'{scratch_1}{scratch_2}{scratch_num}')
+        draw.line(xy=(0, 20 * scale, 115 * scale, 20 * scale), fill=foreground, width=1)
+
+        draw.rectangle(xy=(0, 29 * scale, 20 * scale, 42 * scale), fill=background, outline=foreground)
+        draw.text(xy=(2 * scale, 29 * scale), text=self.get_bios('UFC_COMM1_DISPLAY'), fill=foreground, font=FONT[16 * scale])
 
-    def _draw_common_data(self, draw: ImageDraw, fg: Union[int, Tuple[int, int, int, int]],
-                          bg: Union[int, Tuple[int, int, int, int]], scale: int) -> ImageDraw:
-        # Scrachpad
-        draw.text(xy=(0, 0), fill=fg, font=FONT[16 * scale],
-                  text=f'{self.get_bios("ScratchpadStr1")}{self.get_bios("ScratchpadStr2")}{self.get_bios("ScratchpadNum")}')
-        draw.line(xy=(0, 20 * scale, 115 * scale, 20 * scale), fill=fg, width=1)
-        # comm1
-        draw.rectangle(xy=(0, 29 * scale, 20 * scale, 42 * scale), fill=bg, outline=fg)
-        draw.text(xy=(2 * scale, 29 * scale), text=self.get_bios('COMM1'), fill=fg, font=FONT[16 * scale])
-        # comm2
         offset_comm2 = 44 * scale
-        draw.rectangle(xy=(139 * scale - offset_comm2, 29 * scale, 159 * scale - offset_comm2, 42 * scale), fill=bg, outline=fg)
-        draw.text(xy=(140 * scale - offset_comm2, 29 * scale), text=self.get_bios('COMM2'), fill=fg, font=FONT[16 * scale])
-        # option display 1..5 with cueing
+        draw.rectangle(xy=(139 * scale - offset_comm2, 29 * scale, 159 * scale - offset_comm2, 42 * scale), fill=background, outline=foreground)
+        draw.text(xy=(140 * scale - offset_comm2, 29 * scale), text=self.get_bios('UFC_COMM2_DISPLAY'), fill=foreground, font=FONT[16 * scale])
+
         for i in range(1, 6):
             offset = (i - 1) * 8 * scale
-            draw.text(xy=(120 * scale, offset), fill=fg, font=FONT[11 * scale],
-                      text=f'{i}{self.get_bios(f"OptionCueing{i}")}{self.get_bios(f"OptionDisplay{i}")}')
-        # Fuel Totaliser
-        draw.text(xy=(36 * scale, 29 * scale), text=self.get_bios('IFEI_FUEL_UP'), fill=fg, font=FONT[16 * scale])
+            draw.text(xy=(120 * scale, offset), fill=foreground, font=FONT[11 * scale],
+                      text=f'{i}{self.get_bios(f"UFC_OPTION_CUEING_{i}")}{self.get_bios(f"UFC_OPTION_DISPLAY_{i}")}')
+
+        draw.text(xy=(36 * scale, 29 * scale), text=self.get_bios('IFEI_FUEL_UP'), fill=foreground, font=FONT[16 * scale])
         return draw
 
     def draw_for_lcd_type_1(self, img: Image.Image) -> None:
         """Prepare image for F/A-18C Hornet for Mono LCD."""
-        self._draw_common_data(draw=ImageDraw.Draw(img), fg=255, bg=0, scale=1)
+        self._draw_common_data(draw=ImageDraw.Draw(img), foreground=255, background=0, scale=1)
 
     def draw_for_lcd_type_2(self, img: Image.Image) -> None:
         """Prepare image for F/A-18C Hornet for Color LCD."""
         # todo: maybe add some public members, do some scaling and extract color to Logitech
         green = (0, 255, 0, 255)
         black = (0, 0, 0, 0)
-        draw = self._draw_common_data(draw=ImageDraw.Draw(img), fg=green, bg=black, scale=2)
+        draw = self._draw_common_data(draw=ImageDraw.Draw(img), foreground=green, background=black, scale=2)
         draw.text(xy=(72, 100), text=self.get_bios('IFEI_FUEL_DOWN'), fill=green, font=FONT[32])
 
     def set_bios(self, selector: str, value: str) -> None:
@@ -168,7 +189,8 @@ class FA18Chornet(Aircraft):
         :param selector:
         :param value:
         """
-        if selector in ('ScratchpadStr1', 'ScratchpadStr2', 'COMM1', 'COMM2'):
+        if selector in ('UFC_SCRATCHPAD_STRING_1_DISPLAY', 'UFC_SCRATCHPAD_STRING_2_DISPLAY',
+                        'UFC_COMM1_DISPLAY', 'UFC_COMM2_DISPLAY'):
             value = value.replace('`', '1').replace('~', '2')
         super().set_bios(selector, value)
 
@@ -210,19 +232,24 @@ class F16C50(Aircraft):
         """
         super().__init__(lcd_type)
         self.bios_data: Dict[str, BIOS_VALUE] = {
-            'DED_LINE_1': {'class': 'StringBuffer', 'args': {'address': 0x44FC, 'length': 25}, 'value': str()},
-            'DED_LINE_2': {'class': 'StringBuffer', 'args': {'address': 0x4516, 'length': 25}, 'value': str()},
-            'DED_LINE_3': {'class': 'StringBuffer', 'args': {'address': 0x4530, 'length': 25}, 'value': str()},
-            'DED_LINE_4': {'class': 'StringBuffer', 'args': {'address': 0x454a, 'length': 25}, 'value': str()},
-            'DED_LINE_5': {'class': 'StringBuffer', 'args': {'address': 0x4564, 'length': 25}, 'value': str()}}
+            'DED_LINE_1': {'class': 'StringBuffer', 'args': {'address': 0x4500, 'max_length': 25}, 'value': str()},
+            'DED_LINE_2': {'class': 'StringBuffer', 'args': {'address': 0x451a, 'max_length': 25}, 'value': str()},
+            'DED_LINE_3': {'class': 'StringBuffer', 'args': {'address': 0x4534, 'max_length': 25}, 'value': str()},
+            'DED_LINE_4': {'class': 'StringBuffer', 'args': {'address': 0x454e, 'max_length': 25}, 'value': str()},
+            'DED_LINE_5': {'class': 'StringBuffer', 'args': {'address': 0x4568, 'max_length': 25}, 'value': str()},
+            'IFF_MASTER_KNB': {'class': 'IntegerBuffer', 'args': {'address': 0x444a, 'mask': 0x380, 'shift_by': 0x7}, 'value': int(), 'max_value': 4},
+            'IFF_ENABLE_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x444e, 'mask': 0x3, 'shift_by': 0x0}, 'value': int(), 'max_value': 2},
+            'IFF_M4_CODE_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x444a, 'mask': 0xc00, 'shift_by': 0xa}, 'value': int(), 'max_value': 2},
+            'IFF_M4_REPLY_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x444a, 'mask': 0x3000, 'shift_by': 0xc}, 'value': int(), 'max_value': 2}}
+        self.cycle_buttons = {'IFF_MASTER_KNB': '', 'IFF_ENABLE_SW': '', 'IFF_M4_CODE_SW': '', 'IFF_M4_REPLY_SW': ''}  # type: ignore
 
     def draw_for_lcd_type_1(self, img: Image.Image) -> None:
         """Prepare image for F-16C Viper for Mono LCD."""
         draw = ImageDraw.Draw(img)
         for i in range(1, 6):
             offset = (i - 1) * 8
-            # replace 'o' to degree sign and 'a' with up-down arrow
-            text = str(self.get_bios(f'DED_LINE_{i}')).replace('o', '\u00b0').replace('a', '\u2195')
+            # replace 'o' to degree sign and 'a' with up-down arrow 2195 or black diamond 2666
+            text = str(self.get_bios(f'DED_LINE_{i}')).replace('o', '\u00b0').replace('a', '\u2666')
             draw.text(xy=(0, offset), text=text, fill=255, font=FONT[11])
 
     def draw_for_lcd_type_2(self, img: Image.Image) -> None:
@@ -231,10 +258,30 @@ class F16C50(Aircraft):
         green = (0, 255, 0, 255)
         for i in range(1, 6):
             offset = (i - 1) * 16
-            # replace 'o' to degree sign and 'a' with up-down arrow
-            text = str(self.get_bios(f'DED_LINE_{i}')).replace('o', '\u00b0').replace('a', '\u2195')
+            # replace 'o' to degree sign and 'a' with up-down arrow 2195 or black diamond 2666
+            text = str(self.get_bios(f'DED_LINE_{i}')).replace('o', '\u00b0').replace('a', '\u2666')
             draw.text(xy=(0, offset), text=text, fill=green, font=FONT[22])
 
+    def button_request(self, button: int, request: str = '\n') -> str:
+        """
+        Prepare F-16C Viper specific DCS-BIOS request for button pressed.
+
+        For G13/G15/G510: 1-4
+        For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
+
+        :param button: possible values 1-4
+        :type: int
+        :param request: valid DCS-BIOS command as string
+        :type request: str
+        :return: ready to send DCS-BIOS request
+        :rtype: str
+        """
+        button_map = {1: 'IFF_MASTER_KNB', 2: 'IFF_ENABLE_SW', 3: 'IFF_M4_CODE_SW', 4: 'IFF_M4_REPLY_SW',
+                      9: 'IFF_MASTER_KNB', 10: 'IFF_ENABLE_SW', 14: 'IFF_M4_CODE_SW', 13: 'IFF_M4_REPLY_SW'}
+        button_bios_name = button_map[button]
+        setting = self.get_next_value_for_button(button_bios_name)
+        return super().button_request(button, f'{button_bios_name} {setting}\n')
+
 
 class Ka50(Aircraft):
     def __init__(self, lcd_type: LcdSize) -> None:
@@ -245,16 +292,16 @@ class Ka50(Aircraft):
         """
         super().__init__(lcd_type)
         self.bios_data: Dict[str, BIOS_VALUE] = {
-            'l1_apostr1': {'class': 'StringBuffer', 'args': {'address': 0x1934, 'length': 1}, 'value': str()},
-            'l1_apostr2': {'class': 'StringBuffer', 'args': {'address': 0x1936, 'length': 1}, 'value': str()},
-            'l1_point': {'class': 'StringBuffer', 'args': {'address': 0x1930, 'length': 1}, 'value': str()},
-            'l1_sign': {'class': 'StringBuffer', 'args': {'address': 0x1920, 'length': 1}, 'value': str()},
-            'l1_text': {'class': 'StringBuffer', 'args': {'address': 0x1924, 'length': 6}, 'value': str()},
-            'l2_apostr1': {'class': 'StringBuffer', 'args': {'address': 0x1938, 'length': 1}, 'value': str()},
-            'l2_apostr2': {'class': 'StringBuffer', 'args': {'address': 0x193a, 'length': 1}, 'value': str()},
-            'l2_point': {'class': 'StringBuffer', 'args': {'address': 0x1932, 'length': 1}, 'value': str()},
-            'l2_sign': {'class': 'StringBuffer', 'args': {'address': 0x1922, 'length': 1}, 'value': str()},
-            'l2_text': {'class': 'StringBuffer', 'args': {'address': 0x192a, 'length': 6}, 'value': str()},
+            'PVI_LINE1_APOSTROPHE1': {'class': 'StringBuffer', 'args': {'address': 0x1934, 'max_length': 1}, 'value': str()},
+            'PVI_LINE1_APOSTROPHE2': {'class': 'StringBuffer', 'args': {'address': 0x1936, 'max_length': 1}, 'value': str()},
+            'PVI_LINE1_POINT': {'class': 'StringBuffer', 'args': {'address': 0x1930, 'max_length': 1}, 'value': str()},
+            'PVI_LINE1_SIGN': {'class': 'StringBuffer', 'args': {'address': 0x1920, 'max_length': 1}, 'value': str()},
+            'PVI_LINE1_TEXT': {'class': 'StringBuffer', 'args': {'address': 0x1924, 'max_length': 6}, 'value': str()},
+            'PVI_LINE2_APOSTROPHE1': {'class': 'StringBuffer', 'args': {'address': 0x1938, 'max_length': 1}, 'value': str()},
+            'PVI_LINE2_APOSTROPHE2': {'class': 'StringBuffer', 'args': {'address': 0x193a, 'max_length': 1}, 'value': str()},
+            'PVI_LINE2_POINT': {'class': 'StringBuffer', 'args': {'address': 0x1932, 'max_length': 1}, 'value': str()},
+            'PVI_LINE2_SIGN': {'class': 'StringBuffer', 'args': {'address': 0x1922, 'max_length': 1}, 'value': str()},
+            'PVI_LINE2_TEXT': {'class': 'StringBuffer', 'args': {'address': 0x192a, 'max_length': 6}, 'value': str()},
             'AP_ALT_HOLD_LED': {'class': 'IntegerBuffer', 'args': {'address': 0x1936, 'mask': 0x8000, 'shift_by': 0xf}, 'value': int()},
             'AP_BANK_HOLD_LED': {'class': 'IntegerBuffer', 'args': {'address': 0x1936, 'mask': 0x200, 'shift_by': 0x9}, 'value': int()},
             'AP_FD_LED': {'class': 'IntegerBuffer', 'args': {'address': 0x1938, 'mask': 0x200, 'shift_by': 0x9}, 'value': int()},
@@ -328,19 +375,9 @@ class Ka50(Aircraft):
     def draw_for_lcd_type_1(self, img: Image.Image) -> None:
         """Prepare image for Ka-50 Black Shark for Mono LCD."""
         draw = ImageDraw.Draw(img)
-        text1, text2 = '', ''
-        draw.rectangle(xy=(0, 1, 85, 18), fill=0, outline=255)
-        draw.rectangle(xy=(0, 22, 85, 39), fill=0, outline=255)
-        draw.rectangle(xy=(88, 1, 103, 18), fill=0, outline=255)
-        draw.rectangle(xy=(88, 22, 103, 39), fill=0, outline=255)
-        l1_text = str(self.get_bios('l1_text'))
-        l2_text = str(self.get_bios('l2_text'))
-        if l1_text:
-            text1 = f'{l1_text[-6:-3]}{self.get_bios("l1_apostr1")}{l1_text[-3:-1]}{self.get_bios("l1_apostr2")}{l1_text[-1]}'
-        if l2_text:
-            text2 = f'{l2_text[-6:-3]}{self.get_bios("l2_apostr1")}{l2_text[-3:-1]}{self.get_bios("l2_apostr2")}{l2_text[-1]}'
-        line1 = f'{self.get_bios("l1_sign")}{text1} {self.get_bios("l1_point")}'
-        line2 = f'{self.get_bios("l2_sign")}{text2} {self.get_bios("l2_point")}'
+        for rect_xy in [(0, 1, 85, 18), (0, 22, 85, 39), (88, 1, 103, 18), (88, 22, 103, 39)]:
+            draw.rectangle(xy=rect_xy, fill=0, outline=255)
+        line1, line2 = self._generate_pvi_lines()
         draw.text(xy=(2, 3), text=line1, fill=255, font=FONT[16])
         draw.text(xy=(2, 24), text=line2, fill=255, font=FONT[16])
         self._auto_pilot_switch_1(draw)
@@ -350,23 +387,29 @@ class Ka50(Aircraft):
         draw = ImageDraw.Draw(img)
         green = (0, 255, 0, 255)
         black = (0, 0, 0, 0)
-        text1, text2 = '', ''
-        draw.rectangle(xy=(0, 2, 170, 36), fill=black, outline=green)
-        draw.rectangle(xy=(0, 44, 170, 78), fill=black, outline=green)
-        draw.rectangle(xy=(176, 2, 206, 36), fill=black, outline=green)
-        draw.rectangle(xy=(176, 44, 203, 78), fill=black, outline=green)
-        l1_text = str(self.get_bios('l1_text'))
-        l2_text = str(self.get_bios('l2_text'))
-        if l1_text:
-            text1 = f'{l1_text[-6:-3]}{self.get_bios("l1_apostr1")}{l1_text[-3:-1]}{self.get_bios("l1_apostr2")}{l1_text[-1]}'
-        if l2_text:
-            text2 = f'{l2_text[-6:-3]}{self.get_bios("l2_apostr1")}{l2_text[-3:-1]}{self.get_bios("l2_apostr2")}{l2_text[-1]}'
-        line1 = f'{self.get_bios("l1_sign")}{text1} {self.get_bios("l1_point")}'
-        line2 = f'{self.get_bios("l2_sign")}{text2} {self.get_bios("l2_point")}'
+        for rect_xy in [(0, 2, 170, 36), (0, 44, 170, 78), (176, 2, 206, 36), (176, 44, 203, 78)]:
+            draw.rectangle(xy=rect_xy, fill=black, outline=green)
+        line1, line2 = self._generate_pvi_lines()
         draw.text(xy=(4, 6), text=line1, fill=green, font=FONT[32])
         draw.text(xy=(4, 48), text=line2, fill=green, font=FONT[32])
         self._auto_pilot_switch_2(draw)
 
+    def _generate_pvi_lines(self) -> Tuple[str, str]:
+        text1, text2 = '', ''
+        line1_text = str(self.get_bios('PVI_LINE1_TEXT'))
+        line2_text = str(self.get_bios('PVI_LINE2_TEXT'))
+        if line1_text:
+            l1_apostr1 = self.get_bios("PVI_LINE1_APOSTROPHE1")
+            l1_apostr2 = self.get_bios("PVI_LINE1_APOSTROPHE2")
+            text1 = f'{line1_text[-6:-3]}{l1_apostr1}{line1_text[-3:-1]}{l1_apostr2}{line1_text[-1]}'
+        if line2_text:
+            l2_apostr1 = self.get_bios("PVI_LINE2_APOSTROPHE1")
+            l2_apostr2 = self.get_bios("PVI_LINE2_APOSTROPHE2")
+            text2 = f'{line2_text[-6:-3]}{l2_apostr1}{line2_text[-3:-1]}{l2_apostr2}{line2_text[-1]}'
+        line1 = f'{self.get_bios("PVI_LINE1_SIGN")}{text1} {self.get_bios("PVI_LINE1_POINT")}'
+        line2 = f'{self.get_bios("PVI_LINE2_SIGN")}{text2} {self.get_bios("PVI_LINE2_POINT")}'
+        return line1, line2
+
 
 class F14B(Aircraft):
     def __init__(self, lcd_type: LcdSize) -> None:
@@ -377,10 +420,10 @@ class F14B(Aircraft):
         """
         super().__init__(lcd_type)
         self.bios_data: Dict[str, BIOS_VALUE] = {
-            'RIO_CAP_CLEAR': {'class': 'IntegerBuffer', 'args': {'address': 0x12d4, 'mask': 0x1, 'shift_by': 0x0}, 'value': int()},
-            'RIO_CAP_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x12d2, 'mask': 0x8000, 'shift_by': 0xf}, 'value': int()},
-            'RIO_CAP_NE': {'class': 'IntegerBuffer', 'args': {'address': 0x12d2, 'mask': 0x4000, 'shift_by': 0xe}, 'value': int()},
-            'RIO_CAP_ENTER': {'class': 'IntegerBuffer', 'args': {'address': 0x12d4, 'mask': 0x2, 'shift_by': 0x1}, 'value': int()}}
+            'RIO_CAP_CLEAR': {'class': 'IntegerBuffer', 'args': {'address': 0x12c4, 'mask': 0x4000, 'shift_by': 0xe}, 'value': int()},
+            'RIO_CAP_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x12c4, 'mask': 0x2000, 'shift_by': 0xd}, 'value': int()},
+            'RIO_CAP_NE': {'class': 'IntegerBuffer', 'args': {'address': 0x12c4, 'mask': 0x1000, 'shift_by': 0xc}, 'value': int()},
+            'RIO_CAP_ENTER': {'class': 'IntegerBuffer', 'args': {'address': 0x12c4, 'mask': 0x8000, 'shift_by': 0xf}, 'value': int()}}
 
     def button_request(self, button: int, request: str = '\n') -> str:
         """
